package cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.finance;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.ToString;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.constraints.NotNull;
import java.math.BigDecimal;
import java.util.List;

import static cn.iocoder.yudao.framework.common.util.date.DateUtils.FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND;

/**
 * 活动经费申请 创建 Request VO
 *
 * @author 符溶馨
 */
@Schema(description = "管理后台 - 活动经费申请创建 Request VO")
@Data
@EqualsAndHashCode()
@ToString(callSuper = true)
public class BpmOAFinanceCreateReqVO {

    @Schema(description = "活动主题", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "活动主题不能为空")
    private String title;

    @Schema(description = "活动说明", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "活动说明不能为空")
    private String reason;

    @Schema(description = "活动开始时间", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "活动开始时间不能为空")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String startTime;

    @Schema(description = "活动结束时间", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "活动结束时间不能为空")
    @DateTimeFormat(pattern = FORMAT_YEAR_MONTH_DAY_HOUR_MINUTE_SECOND)
    private String endTime;

    @Schema(description = "活动地址", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "活动地址不能为空")
    private String address;

    @Schema(description = "活动费用预算", requiredMode = Schema.RequiredMode.REQUIRED)
    @NotNull(message = "活动费用预算不能为空")
    private BigDecimal costBudget;

    @Schema(description = "费用明细")
    private List<CostDetail> costDetail;

    @Schema(description = "流程实例编号")
    private String processInstanceId;

    @Schema(description = "状态-参见 bpm_process_instance_result 枚举", requiredMode = Schema.RequiredMode.REQUIRED, example = "1")
    private Integer result;

    @Schema(description = "上传文件", requiredMode = Schema.RequiredMode.REQUIRED)
    private List<UploadUserFile> fileItems;
}
