package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.payment.FactoryDetailVO;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.KeySequence;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.math.BigDecimal;
import java.util.List;

/**
 * 付款申请 DO
 *
 * @author 艾楷
 */
@TableName(value = "bpm_oa_payment", autoResultMap = true)
@KeySequence("bpm_oa_payment") // 用于 Oracle、PostgreSQL、Kingbase、DB2、H2 数据库的主键自增。如果是 MySQL 等数据库，可不写。
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAPaymentDO extends BaseDO {

    /**
     * 付款申请表单主键
     */
    @TableId
    private Long id;

    /**
     * 申请人的用户编号
     * 关联 AdminUserDO 的 id 属性
     */
    private Long userId;

    /**
     * 申请事由
     */
    private String reason;

    /**
     * 付款类型 | 字典值参考 bpm_oa_payment_type
     */
    private Integer type;

    /**
     * 费用产生部门编号
     */
    private Long deptId;

    /**
     * 工厂采购明细数据
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<FactoryDetailVO> factoryDetail;

    /**
     * 项目编号
     */
    private String projectNo;

    /**
     * 采购计划编号
     */
    private Long procureId;

    /**
     * 银行编号
     */
    private Long bankId;

    /**
     * 付款方式 | 1分批付款 3全额付款 4指定金额付款
     */
    private BigDecimal paymentMethod;

    /**
     * 付款总额 | 针对预付款、分期付款情况
     */
    private BigDecimal totalAmount;

    /**
     * 付款比例 | 百分比数字
     */
    private Integer paymentRatio;

    /**
     * 付款金额
     */
    private BigDecimal amount;

    /**
     * 付款公司
     */
    private Long paymentCompany;

    /**
     * 是否后补发票 | 0否 1是
     */
    private Integer isTickets;

    /**
     * 后补票批注
     */
    private String annotations;

    /**
     * 补票状态 | 0待补票 1已补票 2重新补票 3已确认
     */
    private Integer status;

    /**
     * 父级审批
     */
    private Long parentId;

    /**
     * 出差的结果
     * 枚举 {@link BpmProcessInstanceResultEnum}
     * 考虑到简单，所以直接复用了 BpmProcessInstanceResultEnum 枚举，也可以自己定义一个枚举哈
     */
    private Integer result;

    /**
     * 对应的流程编号
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems ;

    /**
     * 后补附件基本信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> laterFileItems;
}
