package cn.iocoder.yudao.module.bpm.dal.dataobject.oa;

import cn.iocoder.yudao.framework.mybatis.core.dataobject.BaseDO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.workorder.BpmOAWorkOrderTrackInfo;
import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import lombok.*;

import java.time.LocalDateTime;
import java.util.List;

/**
 * BPM OA 工单 DO
 *
 * @author 系统
 */
@TableName(value = "bpm_oa_work_order", autoResultMap = true)
@Data
@EqualsAndHashCode(callSuper = true)
@ToString(callSuper = true)
@Builder
@NoArgsConstructor
@AllArgsConstructor
public class BpmOAWorkOrderDO extends BaseDO {

    /**
     * 工单ID，主键
     */
    @TableId
    private Long id;

    /**
     * 工单标题
     */
    private String title;

    /**
     * 工单类型
     * 字典值：work_order_type
     */
    private String type;

    /**
     * 优先级别
     * 1-低，2-中，3-高，4-紧急
     */
    private Integer level;

    /**
     * 工单内容描述
     */
    private String content;

    /**
     * 发起人ID
     */
    private Long fromUserId;

    /**
     * 发起人部门ID
     */
    private Long fromDeptId;

    /**
     * 责任人ID（分配给谁处理）
     */
    private Long assigneeUserId;

    /**
     * 责任部门ID
     */
    private Long assigneeDeptId;

    /**
     * 工单状态
     * 1-待分配，2-处理中，3-已完成，4-已取消，5-已关闭
     */
    private Integer status;

    /**
     * 期望完成时间
     */
    private LocalDateTime expectedTime;

    /**
     * 实际完成时间
     */
    private LocalDateTime completedTime;

    /**
     * 工单处理结果说明
     */
    private String resultDescription;

    /**
     * BPM流程结果
     * 
     * 枚举 {@link BpmProcessInstanceResultEnum}
     */
    private Integer result;

    /**
     * 对应的BPM流程实例ID
     * 
     * 关联 ProcessInstance 的 id 属性
     */
    private String processInstanceId;

    /**
     * 附件信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<UploadUserFile> fileItems;

    /**
     * 工单跟踪记录信息
     */
    @TableField(typeHandler = JacksonTypeHandler.class)
    private List<BpmOAWorkOrderTrackInfo> trackInfo;

} 