package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.UploadUserFile;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.businessCard.BpmOABusinessCardCreateReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOABusinessCardDO;
import cn.iocoder.yudao.module.bpm.dal.mysql.oa.BpmOABusinessCardMapper;
import cn.iocoder.yudao.module.bpm.enums.task.BpmProcessInstanceResultEnum;
import cn.iocoder.yudao.module.bpm.service.task.BpmHistoryProcessInstanceService;
import cn.iocoder.yudao.module.bpm.service.task.BpmProcessInstanceService;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.OA_BUSINESS_CARD_NOT_EXISTS;

/**
 * OA 名片定制 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class BpmOABusinessCardServiceImpl extends BpmOABaseService implements BpmOABusinessCardService{

    /**
     * OA 名片定制对应的流程定义 KEY
     */
    public static final String PROCESS_KEY = "oa_business_card_2";

    @Resource
    private BpmOABusinessCardMapper bpmOABusinessCardMapper;

    @Resource
    private BpmProcessInstanceService processInstanceService;

    @Resource
    private BpmHistoryProcessInstanceService historyProcessInstanceService;

    @Override
    public Long createBusinessCard(Long userId, BpmOABusinessCardCreateReqVO createReqVO) {

        //插入OA 名片定制申请
        BpmOABusinessCardDO bpmOABusinessCardDO = BeanUtils.toBean(createReqVO, BpmOABusinessCardDO.class)
                .setUserId(userId)
                .setResult(BpmProcessInstanceResultEnum.PROCESS.getResult());
        bpmOABusinessCardMapper.insert(bpmOABusinessCardDO);

        // 发起 BPM 流程
        Map<String, Object> processInstanceVariables = new HashMap<>();
        String processInstanceId = processInstanceService.createProcessInstance(userId,
                new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(PROCESS_KEY)
                        .setVariables(processInstanceVariables).setBusinessKey(String.valueOf(bpmOABusinessCardDO.getId())));

        // 将工作流的编号，更新到 OA 现金支出单中
        bpmOABusinessCardMapper.updateById(new BpmOABusinessCardDO()
                .setId(bpmOABusinessCardDO.getId())
                .setProcessInstanceId(processInstanceId));

        // 判断是否为重新发起的流程
        if (createReqVO.getProcessInstanceId() != null && createReqVO.getResult() == 3) {

            historyProcessInstanceService.createHistoryProcessInstance(processInstanceId, createReqVO.getProcessInstanceId());
        }

        List<UploadUserFile> fileItems = createReqVO.getFileItems();
        //这里的逻辑，如果fileItems不为空，且有数据，那么说明是上传了附件的，则需要更工作流文件表对应的实例Id
        if (fileItems != null && !fileItems.isEmpty()) {
            uploadBpmFileProcessInstanceId(processInstanceId, fileItems);
        }
        return bpmOABusinessCardDO.getId();
    }

    @Override
    public BpmOABusinessCardDO getBusinessCard(Long id) {
        return bpmOABusinessCardMapper.selectById(id);
    }

    @Override
    public BpmOABusinessCardDO getByProcessInstanceId(String processInstanceId) {
        return bpmOABusinessCardMapper.selectOne(BpmOABusinessCardDO::getProcessInstanceId, processInstanceId);
    }

    @Override
    public void updateBusinessCardResult(Long id, Integer result) {

        validateLeaveExists(id);
        bpmOABusinessCardMapper.updateById(new BpmOABusinessCardDO().setId(id).setResult(result));
    }

    private void validateLeaveExists(Long id) {
        if (bpmOABusinessCardMapper.selectById(id) == null) {
            throw exception(OA_BUSINESS_CARD_NOT_EXISTS);
        }
    }
}
