package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.expenses.*;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAExpensesDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAExpensesItemDO;

import javax.validation.Valid;
import java.math.BigDecimal;
import java.util.List;

public interface BpmOAExpensesService {

    /**
     * 创建生产开支申请
     *
     * @param userId      用户编号
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createExpenses(Long userId, @Valid BpmOAExpensesCreateReqVO createReqVO);

    /**
     * 更新生产开支申请的状态
     *
     * @param id     编号
     * @param result 结果
     */
    void updateExpensesResult(String processInstanceId, Long id, Integer result);

    /**
     * 获得生产开支申请
     *
     * @param id 编号
     * @return 现金支出申请
     */
    BpmOAExpensesDO getExpenses(Long id);

    /**
     * 获得指定得生产开支明细
     *
     * @param expensesId 生产开支 编号
     * @return 明细
     */
    List<BpmOAExpensesItemDO> getExpensesItem(Long expensesId);

    /**
     * 通过流程实例id获取
     *
     * @param processInstanceId 流程实例编号
     * @return 生产开支对象
     */
    BpmOAExpensesDO getByProcessInstanceId(String processInstanceId);

    /**
     * 转换生产开支详情  用于前端返回值
     * @param expenses 开支对象
     * @return 开支对象
     */
    BpmOAExpensesRespVO convertExpenses(BpmOAExpensesDO expenses);

    /**
     * 获得生产开支申请分页
     * @param pageReqVO 分页信息
     * @return 分页结果
     */
    PageResult<BpmOAExpensesPageRespVO> getExpensesPage(BpmOAExpensesPageReqVO pageReqVO);

    /**
     * 获得开支金额统计
     * @return 统计信息
     */
    BpmOAExpensesTotal getExpensesTotal(BpmOAExpensesPageReqVO pageReqVO);

    /**
     * 提前付款 | 手动插入至支付管理
     * @param id 生产开支id
     */
    void getPayment(Long id);

    /**
     * 更新生产开支 支付状态
     * @param status 支付状态
     * @param amountPaid 支付金额
     */
    void updateExpenses(Long id ,Integer status, BigDecimal amountPaid);

    /**
     * 拒绝付款
     * @param id 生产开支id
     * @param processInstanceId 流程实例编号
     */
    void refused(Long id, String processInstanceId);

    /**
     * 一键支付
     * @param month 月份
     */
    void oneClickPayment(String month);

    /**
     * 转换
     * @param list
     * @return
     */
    List<BpmOAExpensesExportVO> convertExpensesList(List<BpmOAExpensesPageRespVO> list);
}
