package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.invoice.BpmOAInvoiceCreateReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.invoice.BpmOAInvoicePageReqVO;
import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.invoice.BpmOAInvoiceRespVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAInvoiceDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 开票申请 Service 接口
 *
 * @author 符溶馨

 */
public interface BpmOAInvoiceService {

    /**
     * 创建开票申请
     *
     * @param userId 用户编号
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createInvoice(Long userId, @Valid BpmOAInvoiceCreateReqVO createReqVO);

    /**
     * 更新开票申请的状态
     *
     * @param id 编号
     * @param result 结果
     */
    void updateInvoiceResult(Long id, Integer result);

    /**
     * 获得开票申请
     *
     * @param id 编号
     * @return 开票申请
     */
    BpmOAInvoiceDO getInvoice(Long id);

    /**
     * 获得指定开票申请
      * @param processInstanceId 流程实例编号
     * @return 开票申请
     */
    BpmOAInvoiceDO getByProcessInstanceId(String processInstanceId);

    /**
     * 获得开票申请分页列表
     * @param pageReqVO 分页参数
     * @return 开票分页
     */
    PageResult<BpmOAInvoiceRespVO> getInvoicePage(BpmOAInvoicePageReqVO pageReqVO);

    /**
     * 获得开票申请列表
     * @param processInstanceIds 流程实例编号
     * @return 开票申请列表
     */
    List<BpmOAInvoiceRespVO> getListByProcessInstanceIds(List<String> processInstanceIds);
}
