package cn.iocoder.yudao.module.bpm.service.oa;

import cn.iocoder.yudao.module.bpm.controller.admin.oa.vo.overtime.BpmOAOvertimeCreateReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAOvertimeDO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.oa.BpmOAOvertimeItemDO;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.List;

/**
 * 加班申请 Service 接口
 *
 * @author 符溶馨
 */
public interface BpmOAOvertimeService {

    /**
     * 创建加班申请
     *
     * @param userId      用户编号
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOvertime(Long userId, @Valid BpmOAOvertimeCreateReqVO createReqVO);

    /**
     * 更新加班申请的状态
     *
     * @param processInstanceId 流程实例编号
     * @param id                   id
     * @param result               结果
     */
    void updateOvertimeResult(String processInstanceId, Long id, Integer result);

    @Transactional(rollbackFor = Exception.class)
    void handlingOvertime(BpmOAOvertimeDO bpmOAOvertimeDO, List<BpmOAOvertimeItemDO> items, LocalDateTime now);

    /**
     * 获得加班申请
     *
     * @param id 编号
     * @return 加班申请
     */
    BpmOAOvertimeDO getOvertime(Long id);

    /**
     * 获得指定的加班申请
     *
     * @param processInstanceId 流程实例编号
     * @return 加班申请
     */
    BpmOAOvertimeDO getByProcessInstanceId(String processInstanceId);
}
