package cn.iocoder.yudao.module.crm.service.userlivetree;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.crm.controller.admin.userlivetree.vo.UserLiveTreeListVO;
import cn.iocoder.yudao.module.crm.controller.admin.userlivetree.vo.UserLiveTreePageReqVO;
import cn.iocoder.yudao.module.crm.controller.admin.userlivetree.vo.UserLiveTreeSaveReqVO;
import cn.iocoder.yudao.module.crm.dal.dataobject.userlivetree.UserLiveTreeDO;

import javax.validation.Valid;
import java.util.List;

/**
 * crm用户结构树 Service 接口
 *
 * @author 艾楷
 */
public interface UserLiveTreeService {

    /**
     * 创建crm用户结构树
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createUserLiveTree(@Valid UserLiveTreeSaveReqVO createReqVO);

    /**
     * 更新crm用户结构树
     *
     * @param updateReqVO 更新信息
     */
    void updateUserLiveTree(@Valid UserLiveTreeSaveReqVO updateReqVO);

    /**
     * 删除crm用户结构树
     *
     * @param id 编号
     */
    void deleteUserLiveTree(Long id);

    /**
     * 获得crm用户结构树
     *
     * @param id 编号
     * @return crm用户结构树
     */
    UserLiveTreeDO getUserLiveTree(Long id);

    /**
     * 获得crm用户结构树分页
     *
     * @param pageReqVO 分页查询
     * @return crm用户结构树分页
     */
    PageResult<UserLiveTreeListVO> getUserLiveTreePage(UserLiveTreePageReqVO pageReqVO);

    /**
     * 获取列表
     *
     * @return 用户列表
     */
    List<UserLiveTreeListVO> userLiveTreeList();

    /**
     * 通过用户id获取所有子集用户id
     *
     * @param id
     * @return
     */
    List<Long> getItemIdsByUserId(Long id);

    /**
     * 获取所有用户列表
     *
     * @return 用户列表
     */
    List<UserLiveTreeListVO> getUserLiveList();

    /**
     * 获取用户的直接下级
     *
     * @param pid
     * @return
     */
    List<UserLiveTreeDO> getItemByPid(Long pid);
}
