package cn.iocoder.yudao.module.product.service.storecategory;

import cn.iocoder.yudao.module.product.controller.admin.storecategory.vo.StoreProductCategoryCreateReqVO;
import cn.iocoder.yudao.module.product.controller.admin.storecategory.vo.StoreProductCategoryListReqVO;
import cn.iocoder.yudao.module.product.controller.admin.storecategory.vo.StoreProductCategoryUpdateReqVO;
import cn.iocoder.yudao.module.product.dal.dataobject.storecategory.StoreProductCategoryDO;
import com.baomidou.mybatisplus.extension.service.IService;

import javax.validation.Valid;
import java.util.List;

/**
 * 商品分类 Service 接口
 *
 * @author yshop
 */
public interface StoreProductCategoryService extends IService<StoreProductCategoryDO> {

    /**
     * 创建商品分类
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createCategory(@Valid StoreProductCategoryCreateReqVO createReqVO);

    /**
     * 更新商品分类
     *
     * @param updateReqVO 更新信息
     */
    void updateCategory(@Valid StoreProductCategoryUpdateReqVO updateReqVO);

    /**
     * 删除商品分类
     *
     * @param id 编号
     */
    void deleteCategory(Long id);

    /**
     * 获得商品分类
     *
     * @param id 编号
     * @return 商品分类
     */
    StoreProductCategoryDO getCategory(Long id);

    /**
     * 校验商品分类
     *
     * @param id 分类编号
     */
    void validateCategory(Long id);

    /**
     * 获得商品分类的层级
     *
     * @param id 编号
     * @return 商品分类的层级
     */
    Integer getCategoryLevel(Long id);

    /**
     * 获得商品分类列表
     *
     * @param listReqVO 查询条件
     * @return 商品分类列表
     */
    List<StoreProductCategoryDO> getEnableCategoryList(StoreProductCategoryListReqVO listReqVO);

    /**
     * 获得开启状态的商品分类列表
     *
     * @return 商品分类列表
     */
    List<StoreProductCategoryDO> getEnableCategoryList(Integer shopId);

}
