package cn.iocoder.yudao.module.system.service.supplier;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.supplier.vo.SupplierProductPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.supplier.vo.SupplierProductSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.supplier.SupplierProductDO;

import javax.validation.Valid;
import java.util.List;

/**
 * 供应商商品 Service 接口
 *
 * @author 艾楷
 */
public interface SupplierProductService {

    /**
     * 创建供应商商品
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createSupplierProduct(@Valid SupplierProductSaveReqVO createReqVO);

    /**
     * 更新供应商商品
     *
     * @param updateReqVO 更新信息
     */
    void updateSupplierProduct(@Valid SupplierProductSaveReqVO updateReqVO);

    /**
     * 删除供应商商品
     *
     * @param id 编号
     */
    void deleteSupplierProduct(Long id);

    /**
     * 获得供应商商品
     *
     * @param id 编号
     * @return 供应商商品
     */
    SupplierProductDO getSupplierProduct(Long id);

    /**
     * 获得供应商商品分页
     *
     * @param pageReqVO 分页查询
     * @return 供应商商品分页
     */
    PageResult<SupplierProductDO> getSupplierProductPage(SupplierProductPageReqVO pageReqVO);

    /**
     * 获取列表
     *
     * @param supplierId
     * @return
     */
    List<SupplierProductDO> getBySupplierId(Long supplierId);
}
